/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================
 
 Copyright (C) @year@ The Apache Software Foundation. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:
 
 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
 
 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.
 
 4. The names "Jakarta", "Apache Avalon", "Avalon Excalibur", "Avalon
    Framework" and "Apache Software Foundation"  must not be used to endorse
    or promote products derived  from this  software without  prior written
    permission. For written permission, please contact apache@apache.org.
 
 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.
 
 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache 
 Software Foundation, please see <http://www.apache.org/>.
 
*/
package org.apache.avalon.framework.persistab;

import org.apache.avalon.framework.configuration.Configuration;

import java.io.InputStream;
import java.io.OutputStream;

/**
 * <code>Persistable</code> is a interface encapsulating the ability for a
 * Component to expose its current configuration status.  This allows for a
 * Component that is able to change it's configuration state during run-time,
 * and save the results to persistant storage.
 *
 * <p>
 * The contract surrounding the <code>Persistable</code> interface is that
 * it should only be called by the Component's parent.  It may be called any
 * time the configuration state is to be retrieved from the Component.  If a
 * <code>Persistable</code> component contains non-persistable components, the
 * <code>Persistable</code> component must store the original
 * <code>Configuration</code> used to configure the non-persistable component.
 * This way, a system can be restored to its full working state later.
 * </p>
 *
 * @author <a href="mailto:bloritsch@apache.org">Berin Loritsch</a>
 */
public interface Persistable extends Configurable
{
    /**
     * Get a copy of the <code>Configuration</code> object to restore this
     * Component to the state it is currently in.  It can be used to provide
     * "self-healing" configuration persistence or manage sub components
     * dynamically.  "Self-healing" configuration allows you to handle a
     * deprecated configuration schema and persist with the preferred format.
     *
     * <p>
     * NOTE: this is called before Disposable.dispose() and directly before
     *       Persistable.writeData()
     * </p>
     * 
     * @return Configuration object
     */
    Configuration writeConfiguration();
    
    /**
     * Alow a component to load it's working set of data.  The container
     * provides an <code>InputStream</code> to load the information from.
     * The <code>Persistable</code> component must know how to handle an
     * empty stream, initializing the working set of info if the component
     * has never been used before.
     * 
     * <p>
     * NOTE: this is called prior to Initializable.initialize(), but after
     *       everything else.
     * </p>
     */
    void readData(InputStream in);
    
    /**
     * Alow a component to save it's working set of data.  The container
     * provides an <code>OutputStream</code> to save the information to.
     * 
     * <p>
     * NOTE: this is called before Disposable.dispose() and right after
     *       Persistable.writeConfiguration();
     * </p>
     */
    void writeData(OutputStream out);
}
